package com.ruoyi.yh.material.service.impl;

import com.alibaba.fastjson2.JSONArray;
import com.github.pagehelper.PageHelper;
import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.core.redis.enums.RedisLockKey;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.k3cloud.constants.K3FormIds;
import com.ruoyi.common.k3cloud.constants.fileds.K3MaterialField;
import com.ruoyi.common.k3cloud.service.K3Service;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.yh.logImport.domain.LogImportVO;
import com.ruoyi.yh.logImport.domain.enums.LogImportBizType;
import com.ruoyi.yh.logImport.domain.enums.LogImportType;
import com.ruoyi.yh.logImport.service.LogImportService;
import com.ruoyi.yh.logImportDetail.domain.enums.LogImportDetailType;
import com.ruoyi.yh.logImportDetail.service.ILogImportDetailService;
import com.ruoyi.yh.material.domain.Material;
import com.ruoyi.yh.material.domain.MaterialQuery;
import com.ruoyi.yh.material.domain.MaterialVO;
import com.ruoyi.yh.material.mapper.MaterialMapper;
import com.ruoyi.yh.material.service.MaterialConverter;
import com.ruoyi.yh.material.service.MaterialService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 物料Service业务层处理
 *
 * @author ruoyi
 * @date 2024-11-12
 */
@Service
@Slf4j
public class MaterialServiceImpl implements MaterialService
{
    @Autowired
    private MaterialMapper materialMapper;

    @Autowired
    private MaterialConverter materialConverter;

    @Autowired
    private K3Service k3Service;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private RedisLock redisLock;

    @Autowired
    private LogImportService logImportService;

    @Autowired
    private ILogImportDetailService logImportDetailService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    /**
     * 查询物料
     *
     * @param id 物料主键
     * @return 物料
     */
    @Override
    public MaterialVO selectMaterialById(Long id)
    {
        return materialMapper.selectMaterialById(id);
    }

    /**
     * 查询物料列表
     *
     * @param material 物料
     * @return 物料
     */
    @Override
    public List<MaterialVO> selectMaterialList(MaterialQuery material)
    {
        return materialMapper.selectMaterialList(material);
    }

    /**
     * 新增物料
     *
     * @param material 物料
     * @return 结果
     */
    @Override
    public int insertMaterial(Material material)
    {
        return materialMapper.insertMaterial(material);
    }

    /**
     * 修改物料
     *
     * @param material 物料
     * @return 结果
     */
    @Override
    public int updateMaterial(Material material)
    {
        return materialMapper.updateMaterial(material);
    }

    /**
     * 批量删除物料
     *
     * @param ids 需要删除的物料主键
     * @return 结果
     */
    @Override
    public int deleteMaterialByIds(Long[] ids)
    {
        return materialMapper.deleteMaterialByIds(ids);
    }

    /**
     * 删除物料信息
     *
     * @param id 物料主键
     * @return 结果
     */
    @Override
    public int deleteMaterialById(Long id)
    {
        return materialMapper.deleteMaterialById(id);
    }

    @Override
    public void sync(Long operaId) {
        // 新增一个日志
        LogImportVO importLog = logImportService.addDefaultLog(LogImportType.SYNC, LogImportBizType.MATERIAL, operaId);
        ServiceUtil.assertion(importLog == null, "创建同步日志失败");
        Long logId = importLog.getLogId();

        scheduledExecutorService.schedule(() -> {
            // 查询字段
            List<String> fieldKeys = Arrays.asList(
                    K3MaterialField.F_MATERIAL_ID,
                    K3MaterialField.F_NUMBER,
                    K3MaterialField.F_NAME,
                    K3MaterialField.F_SPECIFICATION
            );
            int startRow = 0;
            int limit = 10000;
            int size = 0;

            // 一直查询，直到查到的数据量比limit小，说明查完了
            while(true) {
                // 查询ERP数据
                List<JSONArray> erpList = k3Service.selectList(K3FormIds.MATERIAL, fieldKeys, startRow, limit);

                // 转为PO
                List<Material> materialList = materialConverter.toPoByErpList(fieldKeys, erpList);

                // 保存
                for (Material material : materialList) {
                    scheduledExecutorService.schedule(() -> {
                        this.saveByErpId(material, logId);
                    }, 0, TimeUnit.SECONDS);
                }

                size += materialList.size();

                if (materialList.size() < limit) {
                    logImportService.updateLogToFinished(logId);
                    break;
                } else {
                    startRow += limit;
                }
            }
        }, 0, TimeUnit.SECONDS);

    }

    @Override
    public List<MaterialVO> selectByErpIds(Collection<String> erpIds) {
        if (CollectionUtils.isEmptyElement(erpIds)) {
            return Collections.emptyList();
        }
        MaterialQuery query = new MaterialQuery();
        query.setErpIds(erpIds);
        return this.selectMaterialList(query);
    }

    /**
     * 通过物料编码新增或修改
     */
    private int saveByErpId(Material material, Long logId) {
        if (material == null || StringUtils.isBlank(material.getErpId())) {
            throw new ServiceException("参数错误");
        }
        String lockKey = material.getErpId();
        ServiceUtil.assertion(!redisLock.lock(RedisLockKey.SAVE_MATERIAL, lockKey), "当前物料正在处理中，请稍后重试");

        try {
            // 若数据库中没有,则新增,否则修改
            MaterialVO old = this.selectMaterialByErpId(material.getErpId());
            Integer result = transactionTemplate.execute(status -> {
                if (old == null) {
                    int insert = this.insertMaterial(material);
                    ServiceUtil.assertion(insert != 1, "新增物料失败");
                    int log = logImportDetailService.addResult(logId, LogImportDetailType.ADD, material, null, material.getId());
                    ServiceUtil.assertion(log != 1, "新增日志失败");
                    return insert;
                } else {
                    material.setId(old.getId());
                    int update = this.updateMaterial(material);
                    ServiceUtil.assertion(update != 1, "修改物料失败");
                    int log = logImportDetailService.addResult(logId, LogImportDetailType.UPDATE, material, null, material.getId());
                    ServiceUtil.assertion(log != 1, "新增日志失败");
                    return update;
                }
            });

            return result == null ? 0 : result;
        } catch (Exception e) {
            logImportDetailService.addResult(logId, LogImportDetailType.FAIL, material, e.getMessage(), material.getId());
            return 0;
        } finally {
            redisLock.unlock(RedisLockKey.SAVE_MATERIAL, lockKey);
        }
    }

    private MaterialVO selectMaterialByErpId(String erpId) {
        if (StringUtils.isBlank(erpId)) {
            return null;
        }
        MaterialQuery query = new MaterialQuery();
        query.setErpId(erpId);
        return this.selectOne(query);
    }

    private MaterialVO selectOne(MaterialQuery query) {
        PageHelper.startPage(1,1 );
        List<MaterialVO> list = this.selectMaterialList(query);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }
}
