package com.ruoyi.yh.price.service.impl;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import javax.validation.Validator;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import com.github.pagehelper.PageHelper;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.validate.ValidGroup;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.bean.BeanValidators;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.system.user.domain.SysUserVO;
import com.ruoyi.yh.logImport.domain.LogImportVO;
import com.ruoyi.yh.logImport.domain.enums.LogImportBizType;
import com.ruoyi.yh.logImport.domain.enums.LogImportType;
import com.ruoyi.yh.logImport.service.LogImportService;
import com.ruoyi.yh.logImportDetail.domain.enums.LogImportDetailType;
import com.ruoyi.yh.logImportDetail.service.ILogImportDetailService;
import com.ruoyi.yh.material.utils.MaterialUtil;
import com.ruoyi.yh.price.domain.Price;
import com.ruoyi.yh.price.domain.PriceQuery;
import com.ruoyi.yh.price.domain.PriceVO;
import com.ruoyi.yh.price.domain.dto.PriceImportParams;
import com.ruoyi.yh.price.domain.dto.PricePropertyQuery;
import com.ruoyi.yh.price.domain.dto.PriceVerifyDTO;
import com.ruoyi.yh.price.domain.dto.PriceVerifyPriceDTO;
import com.ruoyi.yh.price.domain.enums.PriceColumn;
import com.ruoyi.yh.price.domain.enums.PriceStatus;
import com.ruoyi.yh.price.domain.vo.PriceSearchConditionVO;
import com.ruoyi.yh.price.mapper.PriceMapper;
import com.ruoyi.yh.price.service.PriceConverter;
import com.ruoyi.yh.price.service.PriceService;
import com.ruoyi.yh.price.service.PriceValidator;
import com.ruoyi.yh.reportProd.domain.dto.ReportProdNewPriceDTO;
import com.ruoyi.yh.reportProd.service.ReportProdConverter;
import com.ruoyi.yh.reportProd.service.ReportProdService;

import lombok.extern.slf4j.Slf4j;

/**
 * 单价Service业务层处理
 *
 * @author ruoyi
 * @date 2024-10-17
 */
@Service
@Slf4j
public class PriceServiceImpl implements PriceService
{
    @Autowired
    private PriceMapper priceMapper;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private ISysDeptService deptService;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private LogImportService logImportService;

    @Autowired
    private ILogImportDetailService logImportDetailService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    protected Validator validator;

    @Autowired
    private ReportProdService reportProdService;

    @Autowired
    private ReportProdConverter reportProdConverter;

    @Autowired
    private PriceConverter priceConverter;

    @Autowired
    private PriceValidator priceValidator;

    /**
     * 查询单价
     *
     * @param priceId 单价主键
     * @return 单价
     */
    @Override
    public PriceVO selectPriceByPriceId(Long priceId)
    {
        return priceMapper.selectPriceByPriceId(priceId);
    }

    /**
     * 查询单价列表
     *
     * @param price 单价
     * @return 单价
     */
    @Override
    public List<PriceVO> selectPriceList(PriceQuery price)
    {
        return priceMapper.selectPriceList(price);
    }

    /**
     * 新增单价
     *
     * @param price 单价
     * @return 结果
     */
    @Override
    public int insertPrice(Price price)
    {
        price.setCreateTime(DateUtils.getNowDate());
        Integer result = transactionTemplate.execute(status -> {
            int insert = priceMapper.insertPrice(price);
            ServiceUtil.assertion(insert != 1, "新增单价失败");

            // 判断是否有重复的单价
            PriceVO vo = selectPriceByPriceId(price.getPriceId());
            priceValidator.validateRepeatEnabled(vo);

            return insert;
        });
        return result == null ? 0 : result;
    }

    /**
     * 修改单价
     *
     * @param data 单价
     * @return 结果
     */
    @Override
    public int updatePrice(Price data)
    {
        PriceVO old = selectPriceByPriceId(data.getPriceId());
        ServiceUtil.assertion(old == null, "待修改的单价不存在");
        ServiceUtil.assertion(!PriceStatus.canEdit().contains(old.getStatus()), "待修改的单价当前状态不允许修改，请刷新后重试");

        data.setUpdateTime(DateUtils.getNowDate());
        Integer result = transactionTemplate.execute(status -> {
            PriceQuery query = new PriceQuery();
            query.setPriceId(data.getPriceId());
            query.setStatusList(PriceStatus.canEdit());
            int update = this.updateByQuery(data, query);
            ServiceUtil.assertion(update != 1, "修改单价失败，当前单价已发生变化，请刷新后重试");

            // 判断是否有重复的单价
            PriceVO vo = selectPriceByPriceId(data.getPriceId());
            priceValidator.validateRepeatEnabled(vo);

            return update;
        });

        return result == null ? 0 : result;
    }

    /**
     * 批量删除单价
     *
     * @param priceIds 需要删除的单价主键
     * @return 结果
     */
    @Override
    public int deletePriceByPriceIds(Long[] priceIds)
    {
        return priceMapper.deletePriceByPriceIds(priceIds);
    }

    /**
     * 删除单价信息
     *
     * @param priceId 单价主键
     * @return 结果
     */
    @Override
    public int deletePriceByPriceId(Long priceId)
    {
        return priceMapper.deletePriceByPriceId(priceId);
    }

    @Override
    public int updateByQuery(Price data, PriceQuery query) {
        if (data == null || query == null) {
            return 0;
        }
        return priceMapper.updateByQuery(data, query);
    }

    @Override
    public int submit(Long priceId) {
        PriceVO old = selectPriceByPriceId(priceId);
        ServiceUtil.assertion(old == null, "待提交的单价不存在");
        ServiceUtil.assertion(!PriceStatus.canSubmit().contains(old.getStatus()), "待提交的单价当前状态不允许提交，请刷新后重试");

        Integer result = transactionTemplate.execute(status -> {
            Price data = new Price();
            data.setStatus(PriceStatus.WAIT_VERIFY.getStatus());
            PriceQuery query = new PriceQuery();
            query.setPriceId(priceId);
            query.setStatusList(PriceStatus.canSubmit());
            int update = this.updateByQuery(data, query);
            ServiceUtil.assertion(update != 1, "提交失败，当前单价状态已发生变化，请刷新后重试");

            return update;
        });

        return result == null ? 0 :result;
    }

    @Override
    public int cancel(Long priceId) {

        PriceVO old = selectPriceByPriceId(priceId);
        ServiceUtil.assertion(old == null, "待取消提交的单价不存在");
        ServiceUtil.assertion(!PriceStatus.canCancel().contains(old.getStatus()), "待提交的单价当前状态不允许取消提交，请刷新后重试");

        Integer result = transactionTemplate.execute(status -> {
            Price data = new Price();
            data.setStatus(PriceStatus.WAIT_SUBMIT.getStatus());
            PriceQuery query = new PriceQuery();
            query.setPriceId(priceId);
            query.setStatusList(PriceStatus.canCancel());
            int update = this.updateByQuery(data, query);
            ServiceUtil.assertion(update != 1, "取消失败，当前单价状态已发生变化，请刷新后重试");

            return update;
        });

        return result == null ? 0 :result;
    }

    @Override
    public int verify(PriceVerifyDTO dto) {
        if (dto == null || CollectionUtils.isEmptyElement(dto.getPriceList()) || dto.getPass() == null) {
            return 0;
        }
        boolean pass = dto.getPass() != null && dto.getPass();
        if (pass) {
            dto.getPriceList().stream().filter(item -> item.getPrice() == null).findFirst()
                    .ifPresent(price -> {
                        throw new ServiceException(String.format("ID为%s的工序单价不允许为空", price.getPriceId()));
                    });
        }

        // 获取旧单价数据
        List<Long> priceIds = CollectionUtils.map(dto.getPriceList(), PriceVerifyPriceDTO::getPriceId);
        List<PriceVO> oldList = selectPriceListByIds(priceIds);

        // 校验
        for (PriceVerifyPriceDTO price : dto.getPriceList()) {
            PriceVO old = oldList.stream().filter(item -> item.getPriceId().equals(price.getPriceId())).findFirst().orElse(null);
            ServiceUtil.assertion(old == null, "待审核ID为%s的单价不存在，请刷新后重试", price.getPriceId());
            ServiceUtil.assertion(!PriceStatus.canVerify().contains(old.getStatus()), "待审核ID为%s的单价当前状态无法审核，请刷新后重试", price.getPriceId());
        }

        Integer result = transactionTemplate.execute(status -> {
            // 修改状态
            List<Price> priceList = priceConverter.toPoByVerify(dto);
            PriceQuery query = new PriceQuery();
            query.setStatusList(PriceStatus.canVerify());
            int update = priceMapper.batchVerify(priceList, query);

            if (pass && update > 0) {
                // 判断是否有重复的单价
                for (PriceVO vo : oldList) {
                    priceValidator.validateRepeatEnabled(vo);
                }

                // 若审核通过，则将通过的单价对应的报表中的单价更新
                List<PriceVO> passList = selectPassListByIds(priceIds);
                List<ReportProdNewPriceDTO> newPriceList = reportProdConverter.toNewPriceDTO(passList);
                reportProdService.batchUpdatePrice(newPriceList);
                
            }

            return update;
        });


        return result == null ? 0 : result;
    }

    private List<PriceVO> selectPassListByIds(List<Long> priceIds) {
        if (CollectionUtils.isEmptyElement(priceIds)) {
            return Collections.emptyList();
        }
        PriceQuery query = new PriceQuery();
        query.setPriceIds(priceIds);
        query.setStatus(PriceStatus.PASS.getStatus());
        return priceMapper.selectPriceList(query);
    }

    @Override
    public int disable(Long priceId) {
        if (priceId == null) {
            return 0;
        }
        PriceVO old = selectPriceByPriceId(priceId);
        ServiceUtil.assertion(old == null, "待禁用的单价不存在，请刷新后重试");
        ServiceUtil.assertion(!PriceStatus.canDisable().contains(old.getStatus()), "待禁用的单价当前状态不允许禁用");
        ServiceUtil.assertion(old.getDisabled() != null && old.getDisabled(), "待禁用的单价已被禁用，请勿重复操作");

        Integer result = transactionTemplate.execute(status -> {
            Price data = new Price();
            data.setDisabled(true);
            data.setDisabledTime(LocalDateTime.now());
            PriceQuery query = new PriceQuery();
            query.setStatusList(PriceStatus.canDisable());
            query.setPriceId(priceId);
            query.setDisabled(false);
            int update = this.updateByQuery(data, query);
            ServiceUtil.assertion(update != 1, "禁用单价失败，当前状态已发生变化，请刷新后重试");

            return update;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int enable(Long priceId) {
        if (priceId == null) {
            return 0;
        }
        PriceVO old = selectPriceByPriceId(priceId);
        ServiceUtil.assertion(old == null, "待启用的单价不存在，请刷新后重试");
        ServiceUtil.assertion(!PriceStatus.canEnable().contains(old.getStatus()), "待启用的单价当前状态不允许禁用");
        ServiceUtil.assertion(old.getDisabled() != null && !old.getDisabled(), "待启用的单价已启用，请勿重复操作");

        Integer result = transactionTemplate.execute(status -> {
            Price data = new Price();
            data.setDisabled(false);
            PriceQuery query = new PriceQuery();
            query.setStatusList(PriceStatus.canEnable());
            query.setPriceId(priceId);
            query.setDisabled(true);
            int update = this.updateByQuery(data, query);
            ServiceUtil.assertion(update != 1, "启用单价失败，当前状态已发生变化，请刷新后重试");

            // 判断是否有重复的单价
            PriceVO vo = selectPriceByPriceId(priceId);
            priceValidator.validateRepeatEnabled(vo);

            return update;
        });

        return result == null ? 0 : result;

    }

    @Override
    public int importPrice(List<PriceVO> priceList, PriceImportParams params, Long operaId) {
        if (CollectionUtils.isEmptyElement(priceList)) {
            throw new ServiceException("导入数据不允许为空");
        }
        boolean updateSupport = params.getUpdateSupport() != null && params.getUpdateSupport();
        boolean verified = params.getVerified() != null && params.getVerified();

        SysUser user = userService.selectUserById(operaId);
        ServiceUtil.assertion(user == null, "操作人不存在");

        // 新增一个日志
        LogImportVO importLog = logImportService.addDefaultLog(LogImportType.IMPORT, LogImportBizType.PRICE, user);
        ServiceUtil.assertion(importLog == null, "创建导入日志失败");

        // 开始异步导入
        scheduledExecutorService.schedule(() -> {
            // 转换数据
            // 部门
            List<SysDept> deptList = deptService.selectDeptListByNames(CollectionUtils.map(priceList, PriceVO::getDeptName));
            // 创建人
            Set<String> nickNames = CollectionUtils.mapToSet(priceList, PriceVO::getCreateBy);
            // 修改人
            nickNames.addAll(CollectionUtils.mapToSet(priceList, PriceVO::getUpdateBy));
            // 审核人
            nickNames.addAll(CollectionUtils.mapToSet(priceList, PriceVO::getVerifyBy));
            List<SysUserVO> userList = userService.selectUserListByNickNames(new ArrayList<>(nickNames));

            // 查询旧数据
            List<PriceVO> oldList = this.selectPriceListByIds(CollectionUtils.map(priceList, PriceVO::getPriceId));
            SysDept defaultDept = new SysDept();
            SysUserVO defaultUser = new SysUserVO();

            for (PriceVO price : priceList) {
                try {
                    // 验证是否存在
                    PriceVO old = oldList.stream().filter(o -> o.getPriceId().equals(price.getPriceId())).findFirst().orElse(null);

                    // 不存在 或者 需要覆盖
                    if (old == null || updateSupport) {
                        // 关联信息
                        price.setDeptId(deptList.stream().filter(d -> d.getDeptName().equals(price.getDeptName())).findFirst().orElse(defaultDept).getDeptId());
                        price.setCreateId(userList.stream().filter(u -> u.getNickName().equals(price.getCreateBy())).findFirst().orElse(defaultUser).getUserId());
                        price.setUpdateId(userList.stream().filter(u -> u.getNickName().equals(price.getUpdateBy())).findFirst().orElse(defaultUser).getUserId());
                        price.setVerifyId(userList.stream().filter(u -> u.getNickName().equals(price.getVerifyBy())).findFirst().orElse(defaultUser).getUserId());
                        if (verified) {
                            price.setStatus(PriceStatus.PASS.getStatus());
                        } else {
                            price.setStatus(PriceStatus.WAIT_VERIFY.getStatus());
                        }
                        price.setCode(MaterialUtil.parseToCode(price.getCategory(), price.getSize(), price.getSurface(), price.getPattern()));

                        // 操作数据库
                        transactionTemplate.execute(status -> {
                            // 存在，则修改，不存在则新增
                            if (old == null) {
                                // 校验
                                BeanValidators.validateWithException(validator, price, ValidGroup.Create.class);
                                // 新增
                                int insert = this.insertPrice(price);
                                ServiceUtil.assertion(insert != 1, "新增失败");
                                logImportDetailService.addResult(importLog.getLogId(), LogImportDetailType.ADD, price, null, price.getPriceId());
                                return insert;
                            } else {
                                // 校验
                                BeanValidators.validateWithException(validator, price, ValidGroup.Update.class);
                                // 修改
                                price.setPriceId(old.getPriceId());
                                int update = this.updatePrice(price);
                                ServiceUtil.assertion(update != 1, "修改失败");
                                logImportDetailService.addResult(importLog.getLogId(), LogImportDetailType.UPDATE, price, null, price.getPriceId());
                                return update;
                            }
                        });
                    }
                    // 不覆盖
                    else {
                        String msg = "单价ID为 " + price.getPriceId() + " 的数据已存在";
                        logImportDetailService.addResult(importLog.getLogId(), LogImportDetailType.FAIL, price, msg, price.getPriceId());
                    }
                } catch (Exception e) {
                    String msg = "单价ID为 " + price.getPriceId() + " 的数据导入失败:" + e.getMessage();
                    log.error(msg, e);
                    logImportDetailService.addResult(importLog.getLogId(), LogImportDetailType.FAIL, price, msg, price.getPriceId());
                }
            }

            // 修改导入状态为处理完成
            logImportService.updateLogToFinished(importLog.getLogId());

        }, 0, TimeUnit.SECONDS);


        return priceList.size();
    }

    @Override
    public List<PriceVO> selectPriceListByIds(List<Long> priceIds) {
        if (CollectionUtils.isEmptyElement(priceIds)) {
            return Collections.emptyList();
        }
        PriceQuery query = new PriceQuery();
        query.setPriceIds(priceIds);
        return priceMapper.selectPriceList(query);
    }

    @Override
    public int logicDel(Long id) {
        if (id == null) {
            return 0;
        }
        PriceVO old = this.selectPriceByPriceId(id);
        ServiceUtil.assertion(old == null, "待删除的单价不存在");
        ServiceUtil.assertion(!PriceStatus.canDel().contains(old.getStatus()), "当前单价状态不允许被删除");

        Price data = new Price();
        data.setDeleted(true);
        PriceQuery query = new PriceQuery();
        query.setPriceId(id);
        query.setStatusList(PriceStatus.canDel());
        query.setDeleted(false);
        return this.updateByQuery(data, query);
    }

    @Override
    public PriceSearchConditionVO selectSearchCondition(PriceQuery query) {
        PriceSearchConditionVO vo = new PriceSearchConditionVO();
        PageHelper.startPage(1, 20);
        vo.setCategoryList(priceMapper.selectColumnCount(query, PriceColumn.category.name()));
        PageHelper.startPage(1, 20);
        vo.setSizeList(priceMapper.selectColumnCount(query, PriceColumn.size.name()));
        PageHelper.startPage(1, 20);
        vo.setSurfaceList(priceMapper.selectColumnCount(query, PriceColumn.surface.name()));
        PageHelper.startPage(1, 20);
        vo.setPatternList(priceMapper.selectColumnCount(query, PriceColumn.pattern.name()));

        return vo;
    }

    @Override
    public PriceVO selectOne(PriceQuery query) {
        PageHelper.startPage(1, 1);
        List<PriceVO> list = priceMapper.selectPriceList(query);
        return CollectionUtils.firstElement(list);
    }

    @Override
    public List<Object> selectProperty(PricePropertyQuery query) {
        if (query.getProp() == null) {
            return Collections.emptyList();
        }
        return priceMapper.selectProperty(query);
    }
}
