package com.ruoyi.yh.report.service.impl;

import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.MathUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.system.user.domain.SysUserQuery;
import com.ruoyi.system.user.domain.SysUserShiftQuery;
import com.ruoyi.system.user.domain.SysUserVO;
import com.ruoyi.yh.price.domain.PriceQuery;
import com.ruoyi.yh.price.domain.PriceVO;
import com.ruoyi.yh.price.domain.enums.PriceStatus;
import com.ruoyi.yh.price.service.PriceService;
import com.ruoyi.yh.prodOrder.domain.enums.ProdOrderErpStatus;
import com.ruoyi.yh.report.domain.ReportVO;
import com.ruoyi.yh.report.domain.bo.ReportBO;
import com.ruoyi.yh.report.domain.enums.ReportStatus;
import com.ruoyi.yh.report.service.ReportValidator;
import com.ruoyi.yh.reportOrderProd.domain.ReportOrderProdVO;
import com.ruoyi.yh.reportOrderProd.domain.bo.ReportOrderProdBO;
import com.ruoyi.yh.reportProd.domain.ReportProdVO;
import com.ruoyi.yh.reportProd.domain.bo.ReportProdBO;
import com.ruoyi.yh.reportUserProd.domain.ReportUserProdVO;
import com.ruoyi.yh.reportUserProd.domain.bo.ReportUserProdBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author wjh
 * 2024/11/8
 */
@Service
public class ReportValidatorImpl implements ReportValidator {

    @Autowired
    private PriceService priceService;

    @Autowired
    private ISysUserService userService;

    @Override
    public void checkPreEdit(ReportBO bo) {
        ServiceUtil.assertion(bo == null, "参数错误");

        // 校验日期
        this.checkReportDate(bo.getReportDate());

        // 校验报表状态
        if (bo.getReportId() != null) {
            ReportVO old = bo.getOld();
            ServiceUtil.assertion(old == null || old.getReportId() == null, "待更新的报表不存在");
            ServiceUtil.assertion(!ReportStatus.canEdit().contains(old.getStatus()), "报表当前状态不允许更新，请刷新后重试");
        }

        // 校验部门
        this.checkDept(bo.getDeptId(), bo.getDept());

        // 校验产量列表
        this.checkProduct(bo.getProductList());

    }

    private void checkReportDate(LocalDate reportDate) {
        ServiceUtil.assertion(reportDate != null && reportDate.isAfter(LocalDate.now()), "报表日期不允许选择未来的日期");
    }

    @Override
    public void afterCheck(ReportVO vo) {
        Long deptId = vo.getDeptId();

        // 工序是否当前部门的工序
        for (ReportProdVO prod : vo.getProductList()) {
            if (prod == null || prod.getPriceId() == null) {
                continue;
            }
            ServiceUtil.assertion(!Objects.equals(prod.getPriceDeptId(), deptId), "工序【%s】不是部门【%s】的工序", prod.getPriceName(), vo.getDeptName());
        }

        // 订单校验
        List<ReportOrderProdVO> orderProdList = vo.getProductList().stream()
                .map(ReportProdVO::getOrderProdList)
                .flatMap(List::stream).collect(Collectors.toList());
        for (ReportOrderProdVO orderProd : orderProdList) {
            if (orderProd == null || orderProd.getOrderId() == null) {
                continue;
            }
            ServiceUtil.assertion(!Objects.equals(orderProd.getOrderDeptId(), deptId), "订单【%s】不是部门【%s】的订单", orderProd.getOrderErpBillNo(), vo.getDeptName());
            ServiceUtil.assertion(!ProdOrderErpStatus.canReport().contains(orderProd.getOrderErpStatus()), "订单【%s】当前状态不允许关联", orderProd.getOrderErpBillNo());
        }

        // 员工是否当天在该部门排班
        List<ReportUserProdVO> userProdList = vo.getProductList().stream()
                .map(ReportProdVO::getUserProdList)
                .flatMap(List::stream).collect(Collectors.toList());
        SysUserShiftQuery userQuery = new SysUserShiftQuery();
        userQuery.setUserIds(CollectionUtils.map(userProdList, ReportUserProdVO::getUserId));
        userQuery.setShiftDate(vo.getReportDate());
        userQuery.setTargetDeptId(vo.getDeptId());
        List<SysUserVO> shiftUserList = userService.selectUserWithShiftList(userQuery);
        for (ReportUserProdVO userProd : userProdList) {
            if (userProd == null || userProd.getUserId() == null) {
                continue;
            }
            SysUserVO shiftUser = shiftUserList.stream()
                    .filter(item -> Objects.equals(item.getUserId(), userProd.getUserId()))
                    .findFirst().orElse(null);
            ServiceUtil.assertion(shiftUser == null, "员工%s在%s没有在部门%s排班", userProd.getUserName(), vo.getReportDate(), vo.getDeptName());
        }
    }

    private void checkProduct(List<ReportProdBO> productList) {
        if (CollectionUtils.isEmptyElement(productList)) {
            return;
        }

        PriceQuery query = new PriceQuery();
        query.setPriceIds(CollectionUtils.map(productList, ReportProdBO::getPriceId));
        query.setNeedScope(true);
        List<PriceVO> priceList = priceService.selectPriceList(query);

        for (ReportProdBO prod : productList) {
            if (prod == null || prod.getPriceId() == null) {
                continue;
            }
            // 校验工序
            PriceVO price = priceList.stream().filter(item -> Objects.equals(item.getPriceId(), prod.getPriceId())).findFirst().orElse(null);
            ServiceUtil.assertion(price == null, "工序%s不存在", prod.getPriceName());
            ServiceUtil.assertion(!PriceStatus.canCheckToReport().contains(price.getStatus()), "工序%s当前状态不允许被使用", price.getName());
            ServiceUtil.assertion(price.getDisabled() == null || price.getDisabled(), "工序%s已被禁用，无法使用", price.getName());


            // 校验是否超出工序的产量（员工、订单）
            BigDecimal userNum = CollectionUtils.sumDecimal(prod.getUserProdList(), ReportUserProdBO::getNum);
            ServiceUtil.assertion(MathUtils.biggerThan(userNum, prod.getNum()), "工序【%s】的用户产量超出工序的产量", prod.getPriceName());
            BigDecimal orderNum = CollectionUtils.sumDecimal(prod.getOrderProdList(), ReportOrderProdBO::getNum);
            ServiceUtil.assertion(MathUtils.biggerThan(orderNum, prod.getNum()), "工序【%s】的订单产量超出工序的产量", prod.getPriceName());
        }

        // 员工是否当前报表可选（在当前报表的部门 or 排班在当前报表部门）
        List<ReportUserProdBO> userProdList = productList.stream().map(ReportProdBO::getUserProdList).flatMap(List::stream).collect(Collectors.toList());
        SysUserQuery userQuery = new SysUserQuery();
        userQuery.setUserIds(CollectionUtils.map(userProdList, ReportUserProdBO::getUserId));
        userQuery.setNeedScope(true);
        List<SysUserVO> userList = userService.selectUserListScope(userQuery);
        for (ReportUserProdBO userProd : userProdList) {
            if (userProd == null || userProd.getUserId() == null) {
                continue;
            }
            // 校验员工
            SysUserVO user = userList.stream().filter(item -> Objects.equals(item.getUserId(), userProd.getUserId())).findFirst().orElse(null);
            ServiceUtil.assertion(user == null, "员工不存在");
        }


    }

    private void checkDept(Long deptId, SysDept dept) {
        if (deptId == null) {
            return;
        }
        // 部门是否当前用户可选
        ServiceUtil.assertion(dept == null, "部门不存在");
    }
}
